#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""Event handler to create the share for TMNAS."""

import os

import ionas.request as request
import ionas.confut as confut
import evh_config as config
import libevh.evhcmd as evhcmd
from fal_info import (FAL_SHARES_PATH, Results, ReqTypes)

EVENTINFO_NAME = "eventinfo"
VOLUME_NAME_PATH = "volume/name"
CATEGORY_ATTR = "category"
ACTION_ATTR = "action"
PREVIOUS_ATTR = "previous"

VOLUME_CATEGORY = "volume"
MOUNT_ACTION = "mount"

SHARES_ADD_REQ_XML = """<shares>
        <share undeletable="true">
            <name>tmadmin</name>
            <desc>ウイルスチェック用</desc>
            <path/>
            <permission>
                <readonly enable="false"/>
                <detailpermission enable="true">
                    <readonlymember>
                        <userlist/>
                        <grouplist/>
                    </readonlymember>
                    <readwritemember>
                        <userlist/>
                        <grouplist/>
                    </readwritemember>
                </detailpermission>
            </permission>
            <shareservice>
                <msshare enable="true">
                    <trash enable="false"/>
                    <hidden enable="true"/>
                    <quota><limit>0.0</limit></quota>
                </msshare>
                <noguest enable="false"/>
            </shareservice>
        </share>
    </shares>
"""


class EvhTmNasShare(evhcmd.EvhCmd):
    """Event handler to create a share for TMNAS."""

    def __send_request(self, target, type, body):
        """Send request entry to "target".""" 

        entry = request.Entry(type, body)
        return self.send_entry(entry, target)

    def __create_share(self):
        """Create a share folder."""

        # Always try create share and ignore error.
        self.__send_request(FAL_SHARES_PATH, ReqTypes.ADD, SHARES_ADD_REQ_XML)

    def __handle_volue_event(self, eventinfo):
        """Handle volume event."""

        action = eventinfo.get_attr_value(ACTION_ATTR)
        is_previous = eventinfo.is_set_to_true(PREVIOUS_ATTR)

        if (eventinfo.get_node(VOLUME_NAME_PATH).get_text() !=
                config.INTERNAL_VOL):

            # Non target events
            entry.set_result(Results.OK)
            return entry

        if action == MOUNT_ACTION:
            if not is_previous:
                self.__create_share()

        entry.set_result(Results.OK)
        return entry

    def __event_handler(self, entry):
        """Handle "event" request."""

        try:
            eventinfo = entry.get().get_node(EVENTINFO_NAME)
            if eventinfo.get_attr_value(CATEGORY_ATTR) == VOLUME_CATEGORY:
                return self.__handle_volue_event(eventinfo)

        except:
            # Non target events
            pass

        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        evhcmd.EvhCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.EVENT, self.__event_handler)


#
# Main
#

if __name__ == "__main__":
    EvhTmNasShare(option=evhcmd.EvhCmdOption()).handle_request()
