#!/usr/bin/perl
#
# chkconfig: 345 99 01
# description: Starts and stops the SPLX daemons used to provide SPLX services.
#
# /etc/init.d/splxcore
#
### BEGIN INIT INFO
# Provides: splxcore
# Required-Start:
# X-UnitedLinux-Should-Start:
# Required-Stop:
# X-UnitedLinux-Should-Stop:
# Default-Start: 3 4 5
# Default-Stop:
# Short-Description: Trend Micro NAS Security core
# Description: Starts and stops the SPLX core daemons used to provide SPLX core services.
### END INIT INFO


use Sys::Syslog qw(:DEFAULT setlogsock);
use POSIX qw(strftime);
use File::Copy;
use File::Path;

# define variables

$uid=$<;

$user_debug_level=0;
$kernel_debug_level=0;
$var_path="";
$user_debug_log_filename="splx_user_debug_log";
$kernel_debug_log_filename="splx_kernel_debug_log";
$MODULE_NAME="splx_service";

#$tmi_cfg_file="/opt/TrendMicro/TMI/TMI.cfg";

$ld_so_conf_file="/etc/ld.so.conf";

$syslog_conf_file = "/etc/syslog-ng/syslog-ng.conf";

$logrotate_file = "/etc/logrotate.d/splx";

$product_root="/opt/TrendMicro/SProtectLinux";
$vsapiapp_root="$product_root/SPLX.vsapiapp";
$module_root="$product_root/SPLX.module";

$ENV{'LD_LIBRARY_PATH'}="$product_root/SPLX.lib";

$tmsplx_xml_file="$product_root/tmsplx.xml";

$locale_file="$product_root/Locale";
$locale_root="$product_root/locale";

$license_file="$product_root/license";

$ARCH=`uname -m`;
chomp($ARCH);

#print $ARCH;

$subsys_path="/var/lock/subsys";

# define commands

if ( -e "/sbin/pidof" ) {
	$pidof="/sbin/pidof";
} elsif ( -e "/bin/pidof") {
	$pidof="/bin/pidof";
} elsif ( -e "/usr/bin/pidof") {
	$pidof="/usr/bin/pidof";
} elsif ( -e "/usr/sbin/pidof") {
	$pidof="/usr/sbin/pidof";
} elsif ( -e "/usr/local/bin/pidof") {
	$pidof="/usr/local/bin/pidof";
} elsif ( -e "/usr/local/sbin/pidof") {
	$pidof="/usr/local/sbin/pidof";
} else {
}
$ldconfig="/sbin/ldconfig";

$insmod="/sbin/insmod";

$rmmod="/sbin/rmmod";

$mknod="/bin/mknod";

$ps="/bin/ps";

# reads debug level from configuration file
use lib qw(/opt/TrendMicro/SProtectLinux/SPLX.lib/pllib/Simple);
use XML::Simple;
$XML::Simple::PREFERRED_PARSER = 'XML::Parser';
$ref = XMLin($tmsplx_xml_file, KeyAttr => {Group=>'Type', P=>'Name'});
$user_debug_level = eval $ref->{Group}->{Configuration}->{P}->{UserDebugLevel}->{Value};
$kernel_debug_level = eval $ref->{Group}->{Configuration}->{P}->{KernelDebugLevel}->{Value};
$var_path = $ref->{Group}->{Configuration}->{P}->{VarPath}->{Value};                
mkpath $var_path, {mode => 0777};

$prefix = "";
if ($user_debug_level == 5) {
	$prefix .= "cd $var_path; ulimit -c unlimited;"
}

sub _system {
	system $prefix . $_[0];
}

# write logs if DEBUG is defined
my $log_file_is_open = 0;	#FALSE
sub open_log{
	if (openlog $MODULE_NAME, '', LOG_LOCAL3) {
		$log_file_is_open = 1;	#TRUE
	}
}

sub write_log{
	my $message=$_[0];

	if ($log_file_is_open) {
		syslog LOG_INFO, $message;
	}

	return 1;	
}

sub close_log{
	if ($log_file_is_open) {
		closelog;
	}
	return 1;
}

# check if this script is run by root

if (defined($uid) && ($uid!=0)) {
	write_log "error: Trend Micro NAS Security must be run by root.";
	print "splxcore must be launched by a root user. Login as root and try again.\n";
	exit 1;
}
# set colors for displaying results

$bootup=$ENV{"BOOTUP"};
$res_col=60;

if (!defined($bootup)) {
	$bootup="color";
}

sub move_to_colon{
	($bootup eq "color") && (print "\033[${res_col}G");
	return 1;
}

sub set_success_color{
	($bootup eq "color") && (print "\033[1;32m");
	return 1;
}

sub set_failure_color{
	($bootup eq "color") && (print "\033[1;31m");
	return 1;
}

sub set_normal_color{
	($bootup eq "color") && (print "\033[0;39m");
	return 1;
}

sub info_success {
    if ( -e "/etc/init.d/functions") {    
        system ("export CONSOLETYPE=vt && . /etc/init.d/functions && \$SETCOLOR_SUCCESS && echo -ne \"" . $_[0] . "\" && \$SETCOLOR_NORMAL");
    }
    else
    {
        set_success_color;
        print $_[0];
        set_normal_color;
    }
}

sub info_failure {
    if ( -e "/etc/init.d/functions") {    
        system ("export CONSOLETYPE=vt && . /etc/init.d/functions && \$SETCOLOR_FAILURE && echo -ne \"" . $_[0] . "\" && \$SETCOLOR_NORMAL");
    }
    else
    {
        set_failure_color;
        print $_[0];
        set_normal_color;
    }
}

# echo success or failure
sub echo_success {
    if (( -e "/etc/init.d/functions") && ($bootup eq "color")){    
        system ("export CONSOLETYPE=vt && . /etc/init.d/functions && \$MOVE_TO_COL && echo -n \"[  \" && \$SETCOLOR_SUCCESS && echo -n \"OK\" && \$SETCOLOR_NORMAL && echo -ne \"  ]\\r\"");
        print "\n"
    }
    else
    {
        ($bootup eq "color") && (&move_to_colon);
        print "[  ";
        
        ($bootup eq "color") && (&set_success_color);
        print "OK";
        
        ($bootup eq "color") && (&set_normal_color);
        print "  ]\n";
        
    }
}

sub echo_failure {
    if ((-e "/etc/init.d/functions") && ($bootup eq "color")){
        system ("export CONSOLETYPE=vt && . /etc/init.d/functions && \$MOVE_TO_COL && echo -n \"[\" && \$SETCOLOR_FAILURE && echo -n \"FAILED\" && \$SETCOLOR_NORMAL && echo -ne \"]\\r\"");
        print "\n";
    }
    else
    {
        ($bootup eq "color") && (&move_to_colon);
        print "[";
        
        ($bootup eq "color") && (&set_failure_color);
        print "FAILED";
        
        ($bootup eq "color") && (&set_normal_color);
        print "]\n";
    }
}

sub echo_notavaliable{
    if ((-e "/etc/init.d/functions") && ($bootup eq "color")){
        system ("export CONSOLETYPE=vt && . /etc/init.d/functions && \$MOVE_TO_COL && echo -n \"[Not available]\" && echo -ne \"]\\r\"");
        print "\n";
    }
    else
    {

        ($bootup eq "color") && (&move_to_colon);
        print "[";
        
        print "Not available";
        
        print "]\n";
    }
}

sub install_logrotate {
	my $mode = $_[0];
	my $logrotate_size = "10M";
	my $logrotate_rotate = "1";

	write_log "step: installing logrotate [$mode].";

	if (length($var_path) == 0) {
		write_log "VarPath is not declared.";
		return 1
	}

	if ($mode == 0) {
		unlink($logrotate_file);
	}
	elsif ($mode == 1) {
		open(LOGROTATE, "> $logrotate_file");
		# user debug log rotate
		print LOGROTATE 
			"$var_path/$user_debug_log_filename {\n"
				."\tcreate 644\n"
				."\tsize $logrotate_size\n"
				."\tnotifempty\n"
				."\tmissingok\n"
				."\trotate $logrotate_rotate\n"
        		."\tpostrotate\n"
		        ."\t\t/etc/init.d/syslog-ng restart\n"
        		."\tendscript\n"
			."}\n";
		# kernel debug log rotate
		print LOGROTATE 
			"$var_path/$kernel_debug_log_filename {\n"
				."\tcreate 644\n"
				."\tsize $logrotate_size\n"
				."\tnotifempty\n"
				."\tmissingok\n"
				."\trotate $logrotate_rotate\n"
        		."\tpostrotate\n"
		        ."\t\t/etc/init.d/syslog-ng restart\n"
        		."\tendscript\n"
			."}\n";
		close(LOGROTATE);
	}

	write_log "Install logrotate successfully.";

	return 0;
}

sub update_syslogconf {
	my $mode = $_[0];
	my $syslog_lines = "";

	write_log "step: updating syslog.conf [$mode].";

	if (length($var_path) == 0) {
		write_log "VarPath is not declared.";
		return 1
	}

	my $syslog_conf_temp = "$var_path/syslog.conf.org";
	my $title_head = "#### TrendMicro TMNAS Syslog Configuration ####";
	my $title_tail = "###############################################";
	my $user_debug_log_line = "local3.*\t\t$var_path/$user_debug_log_filename";
	my $kernel_debug_log_line = "kern.*\t\t$var_path/$kernel_debug_log_filename";

	if ($mode == 0) {
		if (-e $syslog_conf_temp) {
			move("$syslog_conf_temp", "$syslog_conf_file");
		}
		else {
			write_log "Couldn't find $syslog_conf_temp.";
			return 2;
		}
	}
	elsif ($mode == 1) {
		if (-e $syslog_conf_temp) {
			move("$syslog_conf_temp", "$syslog_conf_file");
		}

		$syslog_lines .= "$title_head\n";
		if ($user_debug_level != 0) {
			$syslog_lines .= "$user_debug_log_line\n";
		}
		if ($kernel_debug_level != 0) {
			$syslog_lines .= "$kernel_debug_log_line\n";
		}
		$syslog_lines .= "$title_tail\n";

		copy("$syslog_conf_file", "$syslog_conf_temp");
		open(SYSLOGCONF, ">> $syslog_conf_file");
		print SYSLOGCONF $syslog_lines;
		close(SYSLOGCONF);
	}
	system ("/etc/init.d/syslog-ng restart");
	#system ("killall -HUP syslogd");
	#system ("killall -HUP klogd");
	#system ("klogd -c 5")

	write_log "Updated syslog.conf successfully.";

	return 0;
}

# check status of a program

sub is_running{
	my %property=(
		"running" => 0,
		"pid" => ""
	);
	my $program=$_[0];
	my @pid_list;
	my $pid;
	SWITCH:{
		# splxmod
		($program eq "splxmod") && do {
			open MODULES, "< /proc/modules";
			while (<MODULES>) {
				if (/splxmod/) {
					$property{'running'}=1;
				}
			}
			close MODULES;
			return %property;
			last SWITCH;
		};

		# default
		open PID_OF, "$pidof -o $$ $program |" or return %property;
		while (<PID_OF>) {
			@pid_list=split /\s/, $_;
		}
		close PID_OF;
		foreach $pid (@pid_list) {
			if (kill 0, $pid) {
				if ($property{"pid"} eq "") {
					$property{"pid"}=$pid;
				} else {
					$property{"pid"}.=" $pid";
				}
			}
		}
		if ($property{"pid"} ne "") {
			$property{"running"}=1;
			return %property;
		} else {
			$property{"running"}=0;
			return %property;
		}
	}
}

# show status of programs

sub show_status{
	my $program=$_[0];
	my $running=0;
	my %property=(
		"running" => 0,
		"pid" => ""
	);

 	SWITCH:{
		($program eq "splxmod") && do {
			%property = is_running "splxmod";
			if ($property{"running"}==1) {
				print "$program module is running...\n";
				return 0;
			}
			if ( -e "/var/lock/subsys/splxmod") {
				print "The splxmod module has been loaded successfully, but is not running.\n";
				return 1;
			} else {
				if (! open OS_RELEASE, "< /proc/sys/kernel/osrelease") {
					write_log "ERROR: Cannot open /proc/sys/kernel/osrelease.";
				}
				while (<OS_RELEASE>) {
					chomp($_);
					$kernel_version=$_;
				}
				close OS_RELEASE;


				print "$program module is stopped\n";
				if(-e "$module_root/splxmod-$kernel_version.$ARCH.o")
				{
					return 1;
				}else
				{
					return 0;
				}
			}
			last SWITCH;
		};
		($program eq "entity") && do {
			%property = is_running "entity";
			if ($property{"running"}==1) {
				print "$program (pid ".$property{"pid"}.") is running...\n";
				return 0;
			}
			if (-e "/var/run/entity.pid") {
				open PID_FILE, "< /var/run/entity.pid";
				while (<PID_FILE>) {
					chomp;
					if ($_ ne "") {
						print "The $program is dead, but the pid file still exists. The pid file path is '/var/run/entity.pid'.\n";
						return 1;
					}
					close PID_FILE;
				}
			}
			if ( -e "/var/lock/subsys/entity") {
				print "The $program is dead, but the subsys is locked. The subsys path is '/var/lock/subsys/entity'.\n";
				return 1;
			} else {
				print "$program is stopped\n";
				return 1;
			}
			last SWITCH;
		};
	
		($program eq "vsapiapp") && do {
			%property = is_running $program;
			if ($property{"running"}==1) {
				print "$program (pid ".$property{"pid"}.") is running...\n";
				return 0;
			}

			if (! open OS_RELEASE, "< /proc/sys/kernel/osrelease") {
				write_log "ERROR: Cannot open /proc/sys/kernel/osrelease.";
			}
			while (<OS_RELEASE>) {
				chomp($_);
				$kernel_version=$_;
			}
			close OS_RELEASE;

			if(-e "$module_root/splxmod-$kernel_version.$ARCH.o")
			{
				if (-e "/var/run/$program.pid")
				{
					open PID_FILE, "< /var/run/$program.pid";
					while (<PID_FILE>) {
						print "$program is dead, but the pid file still exists. The pid file path is '/var/run/vsapiapp.pid'.\n";
						close PID_FILE;
						return 1;
					}
				}
				if (-e "/var/lock/subsys/$program")
				{
					print "$program is dead, but the subsys is locked. The subsys path is '/var/lock/subsys/vsapiapp'.\n";
					return 1;
				} else {
					print "$program is stopped\n";
					return 1;
				}
			}else
			{
				print "$program is stopped\n";
				return 0;
			}
		};
		%property = is_running $program;
		if ($property{"running"}==1) {
			print "$program (pid ".$property{"pid"}.") is running...\n";
			return 0;
		}
		if (-e "/var/run/$program.pid") {
			open PID_FILE, "< /var/run/$program.pid";
			while (<PID_FILE>) {
				print "$program dead but pid file exists\n";
				close PID_FILE;
				return 1;
			}
		}
		if (-e "/var/lock/subsys/$program") {
			print "$program dead but subsys locked\n";
			return 1;
		} else {
			print "$program is stopped\n";
			return 1;
		}
	}
}

# kill process

sub kill_proc{
	my $prgm_name=$_[0];
        my @pid_list;
	my $m_num=0;
        my $ret_val=0;
	my $spid='';
        open PID_OF, "$pidof -o $$ $prgm_name |" or return 0;
        while(<PID_OF>){
		chomp;
                @pid_list=split /\s/, $_;
        }
        close PID_OF;
	if(@_ >= 2){
		my $pid_file=$_[1];
		open PID_FD, "<$pid_file";
		while(<PID_FD>){
			chomp;
			$spid=$_;
		}
		close PID_FD;
		$m_num+=grep(/$spid/, @pid_list);
	}
	if($m_num > 0 && $spid != ''){
		$ret_val+=kill 15, $spid;
	}
	else{
		$ret_val+=kill 15, @pid_list;
	}
	return $ret_val > 0 ? 1 : 0;
}

# touch file

sub touch{
	my $filename=$_[0];
	if (-e $_[0]) {
		open TEMP, $filename;
		close TEMP;
	} else {
		open TEMP, "> $filename";
		close TEMP;
	}
}

sub get_locale{
	my $locale_full_path=$_[0];
	$ret = open LOC, "<$locale_full_path";
	if($ret){
		$locale = <LOC>;
		chomp($locale);
	}else{
		$locale = "en_US.UTF-8";
	}
	return $locale;
}

sub set_default_locale_env{
	if(!$ENV{'LC_ALL'}){
		$ENV{'LC_ALL'} = 'C';
	}
}

# read serial number for silent activation
sub get_serial{
    my $lic_file=$_[0];
    my $serial = "";
    $ret = open LIC, "<$lic_file";
    if($ret){
        $serial = <LIC>;
        chomp($serial);
    }
    return $serial;
}


# main program

if ($user_debug_level!=0) {
	open_log;
}
SWITCH:{
	(!defined($ARGV[0])) && do{
		print "Invalid usage of splxcore command.\n";
		print "Valid syntax is: '$0 {start|stop|restart|status}'.\n";
		print "Enter the correct splxcore command and try again.\n";
		exit 1;
	};
	($ARGV[0] eq "stop") && do {
		my $STOP_Entity=0;
		my $STOP_vsapiapp=0;
		my $STOP_splxmod=0;
		my $no_Entity=0;
		my $no_vsapiapp=0;
		my $no_splxmod=0;

		#print "Shutting down Trend Micro NAS Security:\n";
		#write_log "Shutting down Trend Micro NAS Security: ";
		# stop manual/schedule scan
		# shutdown manual scan if needed
		%property=is_running "splx_manual_scan";
		if ($property{"running"}) {
			_system "$vsapiapp_root/splxmain -n > /dev/null 2>&1";
		}

		# shutdown scheduled scan if needed

		# changed to handle case  #TT146013  by errik
		%property=is_running "splx_schedule_s";
		# end
		if ($property{"running"}) {
			_system "$vsapiapp_root/splxmain -t > /dev/null 2>&1";
		}
	
		# shutdown vsapiapp
		print "Shutting down vsapiapp";
		write_log "step: shutting down vsapiapp service.";
		%property=is_running "vsapiapp";
		if ($property{"running"}) {
			$return_value=_system "$vsapiapp_root/splxmain -a > /dev/null 2>&1";
                        $return_value=system "$pidof vsapiapp > /dev/null";
                        if ($return_value==0) {
                                $return_value=system "/usr/bin/killall -9 vsapiapp 2>&1 > /dev/null";
                        } else {
                                $return_value=0;
                        }
                        if ($return_value==0) {
                                unlink "/var/lock/subsys/vsapiapp";
                                write_log "success: vsapiapp service stopped normally.";
                                echo_success;
                                $STOP_vsapiapp=0;
                        } else {
                                write_log "error: vsapiapp service stopped with errors.";
                                echo_failure;
                                $STOP_vsapiapp=1;
                        }
                        #sleep 5;
		} else {
			unlink "/var/lock/subsys/vsapiapp";
			write_log "info: vsapiapp service not running.";
			echo_failure;
			$STOP_vsapiapp=0;
			$no_vsapiapp=1;
		}

		# unload module
		print "Unloading splx kernel module:";
		write_log "step: unloading splx kernel module.";
		%property=is_running "splxmod";
		if ($property{'running'}==0) {
			unlink "/var/lock/subsys/splxmod";
			unlink "/dev/splxdev";
			write_log "info: splx kernel module not loaded.";
			echo_failure;
			$STOP_splxmod=0;
			$no_splxmod=1;
		} else {
			$i=0;
			while ($property{"running"} && $i<15) {
				$return_value=system "$rmmod splxmod > /dev/null 2>&1";
				if ($return_value==1) {
					write_log "Unable to unload splx kernel module. Trend Micro NAS Security will attempt to unload splx kernel module.";
				}
				sleep 1;
				%property=is_running "splxmod";
				$i=$i+1;
			}
			if($property{"running"}){
				write_log "error: Some errors were found while unloading splx kernel module.";
				echo_failure;
				$STOP_splxmod=1;
			}else{
				unlink "/var/lock/subsys/splxmod";
				unlink "/dev/splxdev";
				write_log "success: splx kernel module unloaded.";
				echo_success;
				$STOP_splxmod=0;
			}
		}

		# stop entity
		print "Shutting down entity: ";
		write_log "step: shutting down entity.";
		%property=is_running "entity";
		if ($property{"running"}==0) {
			unlink "/var/lock/subsys/entity";
			write_log "info: entity service not running.";
			echo_failure;
			$STOP_Entity=0;
			$no_Entity=1;
		} else {
			$i=0;
			while ($property{"running"} && $i<15){
				if(! kill_proc "entity", "/var/run/splx.pid"){
					write_log "Unable to stop entity. Trend Micro NAS Security will attempt to stop the entity.";
				}
				sleep 1;
				%property=is_running "entity";
				$i=$i+1;
			}
			if($property{"running"}){
				write_log "warning: Some errors were found while stopping entity. Forcefully terminating it.";
				system "killall -9 entity";
			}
			unlink "/var/lock/subsys/entity";
			write_log "success: entity service stopped.";
			echo_success;
			$STOP_Entity=0;
		}

		# disable jobs of crontab
		_system "$vsapiapp_root/splxmain -b > /dev/null 2>&1";

		#&update_syslogconf(0);
		&install_logrotate(0);

		$STOP=$STOP_Entity+$STOP_vsapiapp+$STOP_splxmod;
		$no_core=$no_Entity+$no_vsapiapp+$no_splxmod;
		if ($STOP==0) {
			if ($no_core==3)
			{
				info_failure("Trend Micro NAS Security core not running.\n");
				unlink "/var/lock/subsys/splxcore";
				write_log "Trend Micro NAS Security core not running.";
				exit 2;
			}
			
			info_success("Trend Micro NAS Security core stopped normally.\n");
			unlink "/var/lock/subsys/splxcore";
			write_log "Trend Micro NAS Security core stopped.";

			exit 0;
		} else {
			info_failure("splxcore stopped with errors. Please contact technical support.\n");
			write_log "Trend Micro NAS Security core stopped with errors.";
			exit 1;
		}

		last SWITCH;
	};
	($ARGV[0] eq "start") && do {
		my $START_Entity=0;
		my $START_vsapiapp=0;
		my $START_splxmod=0;

		&install_logrotate(1);
		#&update_syslogconf(1);
		#print "Starting Trend Micro NAS Security core: \n";
		#write_log "Starting Trend Micro NAS Security core:";
		# remove temporary files left by vsapiapp
		$one = '$1';
		$dollar = '$';
		$awk_regex1 = "'/^V[0-9A-V][0-9A-V][0-9A-V][0-9A-V][0-9A-V][[:print:]][[:print:]][[:print:]][[:print:]][[:print:]][[:print:]]$dollar/ {print $one}'";
		system "cd $product_root/SPLX.tmp/; rm  -f `ls | awk $awk_regex1` ";
		$awk_regex2 = "'/^VS[0-9A-V][0-9A-V][0-9A-V][0-9A-V][0-9A-V][0-9A-V].[0-9A-V][0-9A-V][0-9A-V]$dollar/ {print $one}'";
		system "cd $product_root/SPLX.tmp/; rm  -f `ls | awk $awk_regex2 ` ";


		# check if splxcore is running
		#%property=is_running "entity";
		#if ($property{"running"}) {
		#  set_success_color;
		#  print "Trend Micro NAS Security core running.\n";
		#  set_normal_color;
		#  write_log "error: Trend Micro NAS Security core running. Entity is still running.";
		#  exit 1;
		#}

		# update library path
		#update_library_path;

		# check if tmsplx.xml exists
		if (! -e $tmsplx_xml_file) {
			write_log "error: file $tmsplx_xml_file not exist.";
			echo_failure;
			print "Trend Micro NAS Security core started with errors.\n";
			print "Some errors were found while starting splxcore. See the system log for error description. The default system log file path is /var/log/messages.\n";
			write_log "Trend Micro NAS Security core started with errors.";
			exit 1;
		}

		# co-work with ISVW, IMSS, IWSS
		_system "$product_root/SPLX.util/splxcomp -i > /dev/null 2>&1";


		# for check expire
		$expire_value=_system "$vsapiapp_root/entity -E > /dev/null 2>&1";
		$expire_value >>= 8;

		# locale setting
		my $loc = get_locale $locale_file;
		$ENV{'TM_LOCALE'} = $loc;
		set_default_locale_env;
		chomp($cwd = `pwd`);
		chdir($locale_root) && unlink 'link';
		symlink($loc, 'link');
		chdir($cwd);
		$return_value=_system "$vsapiapp_root/splxmain -C $product_root/SPLX.license/locale/$loc/Corridor.dat > /dev/null 2>&1";
		if ($return_value!=0) {
			write_log "error: failed to change the locale setting.";
			echo_failure;
			print "Unable to change the locale setting of the product. Please contact technical support.\n";
		}

		# create subsys directory
		if (! -d $subsys_path) {
			mkpath($subsys_path, mode=>0777);
		}

		# start Entity
		print "Starting Entity: ";
		write_log "step: starting Entity service.";
		$return_value=0;
		%property=is_running "entity";
		if (!$property{"running"}) {
			if ($user_debug_level!=0) {
				$return_value=_system "$vsapiapp_root/entity -D > $product_root/SPLX.tmp/entity.log 2>&1 &";
			} else {
				$return_value=_system "$vsapiapp_root/entity -D > /dev/null 2>&1 &";
			}
		}
		if ($return_value==0) {
			write_log "step: entity service started.";
			echo_success;
			touch "/var/lock/subsys/entity";
		} else {
			write_log "error: entity service can't be started.";
			echo_failure;
			print "Unable to start the entity. Please contact technical support.\n";
			write_log "Entity service started with errors.";
			$START_Entity=1;
		}

		# try silent activation if the license file exists
		if ($expire_value != 0 && -e $license_file) {
			# check product expiration and prompt to notice the product is expired
			$license_status=_system "$vsapiapp_root/splxmain -E > /dev/null 2>&1";
			$license_status >>= 8;
			if ( $license_status == 1) {
				# try silent activation
				$serial = get_serial $license_file;
				$activate_ret =_system "$vsapiapp_root/splxmain -Q $serial > /dev/null 2>&1";
				$activate_ret >>= 8;
				if ($activate_ret == 0) {
					info_success("The product has been activated successfully with pre-assigned serial number.\n");
					# rename
					if ((system "mv -f $license_file $license_file.bak") != 0) {
						write_log "WARNING: Failed to rename the license file $license_file.bak.";
					}
					# check again
					$expire_value=_system "$vsapiapp_root/entity -E > /dev/null 2>&1";
					$expire_value >>= 8;
				} else {
					write_log "ERROR: Silent activation failed ($activate_ret).";
					info_failure("Failed to activate the product with pre-assigned serial number.\n");
					info_failure("You must activate your product to enable scanning and security updates.\n");
				}
			}
		}

		# check if splx kernel module exist and make a symbolic link
		if (! open OS_RELEASE, "< /proc/sys/kernel/osrelease") {
			write_log "ERROR: Cannot open /proc/sys/kernel/osrelease.";
		}
		while (<OS_RELEASE>) {
			chomp($_);
			$kernel_version=$_;
		}
		close OS_RELEASE;

		if (-e "$module_root/splxmod.o") {
			unlink "$module_root/splxmod.o";
		}
		if (-e "$module_root/splxmod-$kernel_version.$ARCH.o") {
			symlink "$module_root/splxmod-$kernel_version.$ARCH.o", "$module_root/splxmod.o";
		} else {
			write_log "ERROR: No valid KHM present. Failed to find [$module_root/splxmod-$kernel_version.$ARCH.o]";
		}

		if ( $expire_value == 0 ) {
			$kernel_map_file = "/boot/System.map-$kernel_version";
			if (! -f "$kernel_map_file" ) {
				write_log "Could not find $kernel_map_file.";
				@v_list=split /-/, $kernel_version;
				$kernel_map_file = "/boot/System.map-@v_list[0]";
				if (! -f "$kernel_map_file" ) {
					write_log "Could not find $kernel_map_file.";
					if ( -f "/proc/kallsyms" ) {
						$kernel_map_file = "/proc/kallsyms";
					}
				}
			}
			write_log "Found \"$kernel_map_file\". Use it instead.";

			# installs splx kernel module
			print "Loading splx kernel module: ";
			write_log "step: loading splx kernel module.";

			$splxmod_addr="0";
			if (! open SYS_MAP, "< $kernel_map_file") {
				write_log "ERROR: Cannot open $kernel_map_file.";
			}
			while (<SYS_MAP>) {
				chomp($_);
				if ($_=~/ sys_call_table/) {
					@tk_list=split / /, $_;
					if ($tk_list[2] eq "sys_call_table") {
						$splxmod_addr=$tk_list[0];
						last;
					}
				}
			}
			close SYS_MAP;


			$splxmod_execve_addr="0";
			if (! open SYS_MAP, "< $kernel_map_file") {
				write_log "ERROR: Cannot open $kernel_map_file.";
			}
			while (<SYS_MAP>) {
				chomp($_);
				if ($_=~/ do_execve/) {
					@tk_list=split / /, $_;
					if ($tk_list[2] eq "do_execve") {
						$splxmod_execve_addr=$tk_list[0];
						last;
					}
				}
			}
			close SYS_MAP;
			

			if ($ARCH eq "x86_64"){
				$splxmod_ret_addr="0";
				if (! open SYS_MAP, "< $kernel_map_file") {
					write_log "ERROR: Cannot open $kernel_map_file.";
				}
				while (<SYS_MAP>) {
					chomp($_);
					if ($_=~/ int_ret_from_sys_call/) {
						@tk_list=split / /, $_;
						if ($tk_list[2] eq "int_ret_from_sys_call") {
							$splxmod_ret_addr=$tk_list[0];
							last;
						}
					}
				}
				close SYS_MAP;							
				
				$splxmod_ia32_addr="0";
				if (! open SYS_MAP, "< $kernel_map_file") {
					write_log "ERROR: Cannot open $kernel_map_file.";
				}
				while (<SYS_MAP>) {
					chomp($_);
					if ($_=~/ ia32_sys_call_table/) {
						@tk_list=split / /, $_;
						if ($tk_list[2] eq "ia32_sys_call_table") {
							$splxmod_ia32_addr=$tk_list[0];
							last;
						}
					}
				}
				close SYS_MAP;
				
				$splxmod_compat_do_execve_addr="0";
				if (! open SYS_MAP, "< $kernel_map_file") {
					write_log "ERROR: Cannot open $kernel_map_file.";
				}
				while (<SYS_MAP>) {
					chomp($_);
					if ($_=~/ compat_do_execve/) {
						@tk_list=split / /, $_;
						if ($tk_list[2] eq "compat_do_execve") {
							$splxmod_compat_do_execve_addr=$tk_list[0];
							last;
						}
					}
				}
				close SYS_MAP;														
			}

			%property=is_running "splxmod";
			$return_value=0;
			if (!$property{"running"})
			{
				if (! -e "/var/lock/subsys/splxmod" && -e "$module_root/splxmod-$kernel_version.$ARCH.o") {
					if ($ARCH eq "x86_64"){
						$Insmod_Command = "$insmod $module_root/splxmod.o splxmod_addr=0x$splxmod_addr splxmod_execve_addr=0x$splxmod_execve_addr splxmod_ret_addr=0x$splxmod_ret_addr splxmod_ia32_addr=0x$splxmod_ia32_addr splxmod_compat_do_execve_addr=0x$splxmod_compat_do_execve_addr";						
						}
						else 
						{
	                                         $Insmod_Command = "$insmod $module_root/splxmod.o splxmod_addr=0x$splxmod_addr splxmod_execve_addr=0x$splxmod_execve_addr";
						}
#						print "\n";
#					        print $Insmod_Command;
#					        print "\n";
						$return_value=system "$Insmod_Command >/dev/null 2>&1";
						if ($return_value==0) {
							write_log "success: splx kernel module loaded.";
							echo_success;
							touch "/var/lock/subsys/splxmod";
						} else {
							write_log "error: executing [$Insmod_Command] failed";
							write_log "error: splx kernel module can't be loaded.";
							echo_failure;
							print "Unable to load Kernel module. Please contact technical support.\n";
							write_log "Kernel module can't be loaded.";
							$START_splxmod=1;
						}
				} elsif(! -e "$module_root/splxmod-$kernel_version.$ARCH.o") {
	    	     			echo_notavaliable;
	                                print "\nError:   Kernel Hook Module (KHM) for this Linux kernel version is not\navailable. Check if the KHM for your  Linux kernel version is released\non the Trend Micro website at  \"http://www.trendmicro.com/en/products/\nfile-server/sp-linux/use/kernel.htm\".Or,follow the instructions in the\nINSTALL file in \"/opt/TrendMicro/SProtectLinux/SPLX.module/src/module\"\nto build the KHM for your Linux kernel version.\n\n";
	         			write_log "Kernel module can't be loaded.";

	      		} else {
					write_log "error: /var/lock/subsys/splxmod present. KHM already loaded?";
				}
			}else
			{
				write_log "success: splx kernel module loaded.";
				echo_success;
			}

			# generate the splx device file
			if ( system("/sbin/lsmod | grep splxmod > /dev/null 2>&1") == 0) {
				write_log "step: creating device file.";
				if (-e "/dev/splxdev") {
					unlink "/dev/splxdev";
				}
				if (! open DEVICES, "/proc/devices") {
					write_log "ERROR: Cannot open /proc/devices.";
				}
				while (<DEVICES>) {
					if (/splxdev/) {
						($device_number)=split /\s/, $_,0;
					}
				}
				close DEVICES;

				if (defined($device_number)) {
					$return_value=system "$mknod -m=640 /dev/splxdev c $device_number 0 > /dev/null 2>&1";
				} else {
					write_log "error: no such device. Make sure KHM has been loaded successfully.";
					if( -e "$module_root/splxmod-$kernel_version.$ARCH.o")
					{
						$START_splxmod=1;
					}
				}

				if (defined($return_value)&&($return_value==0)) {
					write_log "success: device file created.";
				} else {
					write_log "error: device file can't be created.";
					print "Unable to create device file. Please contact technical support.\n";
					write_log "Device file can't be created.";
					if(-e "$module_root/splxmod-$kernel_version.$ARCH.o" )
					{
						$START_splxmod=1;
					}
				}
			}

			# start vsapiapp
			print "Starting vsapiapp: ";
			write_log "step: starting vsapiapp service.";
			$check_pattern=0;
			$check_pattern=_system "$vsapiapp_root/virus_type_finder > /dev/null 2>&1";
			if($check_pattern==0)
			{
				$return_value=0;
				%property=is_running "vsapiapp";
				if (!$property{"running"}) {
					$return_value=_system "$vsapiapp_root/splxmain -e -D > /dev/null 2>&1";
				}
				if ($return_value==0) {
					write_log "step: vsapiapp service started.";
					echo_success;
					touch "/var/lock/subsys/vsapiapp";
				} else {
					write_log "error: vsapiapp service can't be started.";
					echo_failure;
					print "Unable to start the Scan Engine (Vsapiapp) service. Please contact technical support.\n";
					write_log "Vsapiapp service can't be started.";
					$START_vsapiapp=1;
				}
			}
			else
			{
				write_log "error: vsapiapp service can't be started.";
				echo_failure;
				print "Real-time scan was unable to load the scan engine or pattern files. The scan engine or pattern files may be missing or corrupted. Please click \"Update Now\" in the Summary screen to download the latest components.\n";
				write_log "Vsapiapp service can't be started.";
				$START_vsapiapp=1;
			}
		} else {
			print "Loading splx kernel module: ";
			echo_failure;
			print "Starting vsapiapp: ";
			echo_failure;
    	
			write_log "Please make sure the product has been activated. KHM splxmod.o will not be loaded."
		}
		
		# enable jobs of crontab
		_system "$vsapiapp_root/splxmain -c > /dev/null 2>&1";

		$START=$START_Entity+$START_vsapiapp+$START_splxmod;
		if ($START==0) {
			touch "/var/lock/subsys/splxcore";
			info_success("Trend Micro NAS Security core started.\n");
			write_log "Trend Micro NAS Security core started.";
		} else {
			info_failure( "Some errors were found while starting splxcore. See the system log for error description. The default system log file path is /var/log/messages.\n");
			write_log "Trend Micro NAS Security core started with errors.";
			exit 1;
		}

		last SWITCH;
	};
		
	($ARGV[0] eq "restart") && do {
		system "$0 stop";
		system "$0 start";
		last SWITCH;
	};
	($ARGV[0] eq "status") && do {
		my $STATUS_Entity=0;
		my $STATUS_vsapiapp=0;
		my $STATUA_splxmod=0;
		$STATUA_splxmod=show_status "splxmod";
		$STATUS_vsapiapp=show_status "vsapiapp";
		$STATUS_Entity=show_status "entity";
		$STATUS=$STATUS_Entity+$STATUS_vsapiapp+$STATUS_splxmod;
		if ($STATUS==0) {
			exit 0;
		} else {
			exit 1;
		}
		last SWITCH;
	};
    print "Invalid usage of splxcore command.\n";
    print "Valid syntax is: '$0 {start|stop|restart|status}'.\n";
    print "Enter the correct splxcore command and try again.\n";
}

if ($user_debug_level!=0) {
	close_log;
}

exit 0;
