#!/usr/bin/perl
#
# chkconfig: 345 99 01
# description: Starts and stops the SPLX daemons used to provide SPLX services.
#
# /etc/init.d/splxhttpd
#
### BEGIN INIT INFO
# Provides: splxhttpd
# Required-Start:
# X-UnitedLinux-Should-Start:
# Required-Stop:
# X-UnitedLinux-Should-Stop:
# Default-Start: 3 4 5
# Default-Stop:
# Short-Description: Trend Micro NAS Security httpd
# Description: Starts and stops the SPLX httpd daemons used to provide SPLX httpd services.
### END INIT INFO

use Sys::Syslog qw(:DEFAULT setlogsock);
use POSIX qw(strftime);
use File::Path;

# define variables

$uid=$<;

$debuglevel=0;
$MODULE_NAME="splx_service";

#$tmi_cfg_file="/opt/TrendMicro/TMI/TMI.cfg";

$ld_so_conf_file="/etc/ld.so.conf";

$product_root="/opt/TrendMicro/SProtectLinux";

$ENV{'LD_LIBRARY_PATH'}="$product_root/SPLX.lib";

$splx_log_file="$product_root/SPLX.tmp/splx.log";

$tmsplx_xml_file="$product_root/tmsplx.xml";


$httpd_prog="$product_root/SPLX.httpd/bin/splxhttpd";

$webui_root="$product_root/SPLX.WebUI";

$locale_file="$product_root/Locale";

$locale_root="$product_root/locale";

$locale_link="locale/link";

$subsys_path="/var/lock/subsys";

# define commands

if( -e "/sbin/pidof" )
{
        $pidof="/sbin/pidof";
}elsif ( -e "/bin/pidof")
{
        $pidof="/bin/pidof";
}elsif ( -e "/usr/bin/pidof")
{
        $pidof="/usr/bin/pidof";
}elsif ( -e "/usr/sbin/pidof")
{
        $pidof="/usr/sbin/pidof";
}elsif ( -e "/usr/local/bin/pidof")
{
        $pidof="/usr/local/bin/pidof";
}elsif ( -e "/usr/local/sbin/pidof")
{
        $pidof="/usr/local/sbin/pidof";
}else
{
}
$ldconfig="/sbin/ldconfig";

$insmod="/sbin/insmod";

$rmmod="/sbin/rmmod";

$mknod="/bin/mknod";
$ps="/bin/ps";
# reads debug level from configuration file

use lib qw(/opt/TrendMicro/SProtectLinux/SPLX.lib/pllib/Simple);
use XML::Simple;
$XML::Simple::PREFERRED_PARSER = 'XML::Parser';
$ref = XMLin($tmsplx_xml_file, KeyAttr => {Group=>'Type', P=>'Name'});
$debuglevel = eval $ref->{Group}->{Configuration}->{P}->{UserDebugLevel}->{Value};
$var_path = $ref->{Group}->{Configuration}->{P}->{VarPath}->{Value};
if ($var_path eq undef) {
  $var_path = "/opt/TrendMicro/SProtectLinux/SPLX.httpd/logs";
}
$httpd_pid_file=$var_path . "/httpd/logs/splxhttpd.pid";
$httpd_conf_file='/opt/TrendMicro/SProtectLinux/SPLX.httpd/conf/splxhttpd.conf';

$logrotate_file = '/etc/logrotate.d/splxhttpd';
$splx_httpd_log_path = $var_path . "/httpd/logs/";
$logrotate_rotate = '5';

# write logs if DEBUG is defined
my $log_file_is_open = 0; #FALSE
sub open_log{
  if (openlog $MODULE_NAME, '', LOG_LOCAL3) {
    $log_file_is_open = 1; #TRUE
  }
}

sub write_log {
  my $message=$_[0];

  if ($log_file_is_open) {
    syslog LOG_INFO, $message;
  }
  return 1;	
}

sub close_log{
  if ($log_file_is_open) {
    closelog;
  }
  return 1;
}

sub install_logrotate {
  my $mode = $_[0];
  my $logrotate_size = "10M";

  write_log "step: installing logrotate [$mode].";

  if (length($var_path) == 0) {
    write_log "VarPath is not declared.";
    return 1
  }

  if ($mode == 0) {
    unlink($logrotate_file);
  }
  elsif ($mode == 1) {
    open(LOGROTATE, "> $logrotate_file");
    # splxhttpd
    print LOGROTATE
      "$splx_httpd_log_path" . "*log {\n"
        ."\tcreate 644\n"
        ."\tnotifempty\n"
        ."\tmissingok\n"
        ."\trotate $logrotate_rotate\n"
        ."\tsharedscripts\n"
        ."\tpostrotate\n"
        ."\t\tkill -HUP `cat $httpd_pid_file`\n"
        ."\t\t/etc/init.d/syslog-ng restart\n"
        ."\tendscript\n"
      ."}\n";
    close(LOGROTATE);
  }

  write_log "Install logrotate successfully.";

  return 0;
}


# check if this script is run by root

if (defined($uid) && ($uid!=0)){
	write_log "error: Trend Micro NAS Security must be run by root.";
	print "splxhttpd must be launched by a root user. Login as root and try again.\n";
	exit 1;
}
# set colors for displaying results

$bootup=$ENV{"BOOTUP"};
$res_col=60;

if (!defined($bootup)) {
	$bootup="color";
}

sub move_to_colon{
	($bootup eq "color") && (print "\033[${res_col}G");
	return 1;
}

sub set_success_color{
	($bootup eq "color") && (print "\033[1;32m");
	return 1;
}

sub set_failure_color{
	($bootup eq "color") && (print "\033[1;31m");
	return 1;
}

sub set_normal_color{
	($bootup eq "color") && (print "\033[0;39m");
	return 1;
}

sub info_success {
    if ( -e "/etc/init.d/functions") {    
        system ("export CONSOLETYPE=vt && . /etc/init.d/functions && \$SETCOLOR_SUCCESS && echo -ne \"" . $_[0] . "\" && \$SETCOLOR_NORMAL");
    }
    else
    {
        set_success_color;
        print $_[0];
        set_normal_color;
    }
}

sub info_failure {
    if ( -e "/etc/init.d/functions") {    
        system ("export CONSOLETYPE=vt && . /etc/init.d/functions && \$SETCOLOR_FAILURE && echo -ne \"" . $_[0] . "\" && \$SETCOLOR_NORMAL");
    }
    else
    {
        set_failure_color;
        print $_[0];
        set_normal_color;
    }
}

# echo success or failure
sub echo_success {
    if (( -e "/etc/init.d/functions") && ($bootup eq "color")){    
        system ("export CONSOLETYPE=vt && . /etc/init.d/functions && \$MOVE_TO_COL && echo -n \"[  \" && \$SETCOLOR_SUCCESS && echo -n \"OK\" && \$SETCOLOR_NORMAL && echo -ne \"  ]\\r\"");
        print "\n"
    }
    else
    {
        ($bootup eq "color") && (&move_to_colon);
        print "[  ";
        
        ($bootup eq "color") && (&set_success_color);
        print "OK";
        
        ($bootup eq "color") && (&set_normal_color);
        print "  ]\n";
        
    }
}

sub echo_failure {
    if ((-e "/etc/init.d/functions") && ($bootup eq "color")){
        system ("export CONSOLETYPE=vt && . /etc/init.d/functions && \$MOVE_TO_COL && echo -n \"[\" && \$SETCOLOR_FAILURE && echo -n \"FAILED\" && \$SETCOLOR_NORMAL && echo -ne \"]\\r\"");
        print "\n";
    }
    else
    {
        ($bootup eq "color") && (&move_to_colon);
        print "[";
        
        ($bootup eq "color") && (&set_failure_color);
        print "FAILED";
        
        ($bootup eq "color") && (&set_normal_color);
        print "]\n";
    }
}

# check status of a program

sub is_running{
	my %property=(
			"running" => 0,
			"pid" => ""
			);
	my $program=$_[0];
	my @pid_list;
	my $pid;
	SWITCH:{
		# splxmod
		($program eq "splxmod") && do {
			open MODULES, "< /proc/modules";
			while(<MODULES>){
				if(/splxmod/){
					$property{'running'}=1;
				}
			}
			close MODULES;
			return %property;
			last SWITCH;
		};

		# default
		open PID_OF, "$pidof -o $$ $program |" or return %property;
		while(<PID_OF>){
			@pid_list=split /\s/, $_;
		}
		close PID_OF;
		foreach $pid (@pid_list){
			if(kill 0, $pid){
				if($property{"pid"} eq ""){
					$property{"pid"}=$pid;
				}
				else{
					$property{"pid"}.=" $pid";
				}
			}
		}
		if($property{"pid"} ne ""){
			$property{"running"}=1;
			return %property;
		}
		else{
			$property{"running"}=0;
			return %property;
		}
	}
}

# show status of programs

sub show_status{
	my $program=$_[0];
	my $running=0;
	my %property=(
			"running" => 0,
			"pid" => ""
			);

	SWITCH:{
		($program eq "splxmod") && do {
			%property = is_running "splxmod";
			if($property{"running"}==1){
				print "$program module is running...\n";
				return 0;
			}
			if( -e "/var/lock/subsys/splxmod"){
				print "$program module has been loaded but is not running...\n";
				return 1;
			}
			else{
				print "$program module is stopped\n";
				return 1;
			}
			last SWITCH;
			};
		($program eq "entity") && do {
			%property = is_running "entity";
			if($property{"running"}==1){
				print "$program (pid ".$property{"pid"}.") is running...\n";
				return 0;
			}
			if(-e "/var/run/entity.pid"){
				open PID_FILE, "< /var/run/entity.pid";
				while(<PID_FILE>){
					chomp;
					if($_ ne ""){
						print "$program dead but pid file exists\n";
						return 1;
					}
					close PID_FILE;
				}
			}
			if( -e "/var/lock/subsys/entity.pid"){
				print "$program dead but subsys locked\n";
				return 1;
			}
			else{
				print "$program is stopped\n";
				return 1;
			}
			last SWITCH;
			};
		%property = is_running $program;
		if($property{"running"}==1){
			print "$program (pid ".$property{"pid"}.") is running...\n";
			return 0;
		}
		if(-e "/var/run/$program.pid"){
			open PID_FILE, "< /var/run/$program.pid";
			while(<PID_FILE>){
				print "$program is dead, but the pid file still exists. The pid file path is '/var/run/splxhttpd.pid'.\n";
				close PID_FILE;
				return 1;
			}
		}
		if(-e "/var/lock/subsys/$program"){
			print "$program is dead, but the subsys is locked. The subsys path is '/var/lock/subsys/splxhttpd'.\n";
			return 1;
		}
		else{
			print "$program is stopped\n";
			return 1;
		}
	}
}

# kill process

sub kill_proc{
	my $prgm_name=$_[0];
        my @pid_list;
	my $m_num=0;
        my $ret_val=0;
	my $spid='';
        open PID_OF, "$pidof -o $$ $prgm_name |" or return 0;
        while(<PID_OF>){
		chomp;
                @pid_list=split /\s/, $_;
        }
        close PID_OF;
	if(@_ >= 2){
		my $pid_file=$_[1];
		open PID_FD, "<$pid_file";
		while(<PID_FD>){
			chomp;
			$spid=$_;
		}
		close PID_FD;
		$m_num+=grep(/$spid/, @pid_list);
	}
	if($m_num > 0 && $spid != ''){
		$ret_val+=kill 15, $spid;
	}
	else{
		$ret_val+=kill 15, @pid_list;
	}
	return $ret_val > 0 ? 1 : 0;
}

# touch file

sub touch{
	my $filename=$_[0];
	if(-e $_[0]){
		open TEMP, $filename;
		close TEMP;
	}
	else{
		open TEMP, "> $filename";
		close TEMP;
	}
}

sub get_locale{
	my $locale_full_path=$_[0];
	$ret = open LOC, "<$locale_full_path";
	if($ret){ 
		$locale = <LOC>;
		chomp($locale);
	}else{
		$locale = "en_US.UTF-8";
	}
	return $locale;
}

sub set_default_locale_env{
	if(!$ENV{'LC_ALL'}){
		$ENV{'LC_ALL'} = 'C';
	}
}

sub make_mui_links{
	my $loc=$_[0];
	my $ret=0;
	my @mui_dirs = ("html");
	foreach $dir ( @mui_dirs ){
		#remove previous link if exists
		my $mui_link = "$webui_root/$dir/$locale_link"; 
		if( -l $mui_link ){
			$ret = unlink $mui_link;
			if($ret == 0){
				return 0;
			}
		}
		#make symbolic link for mui
		$ret = symlink "$webui_root/$dir/locale/$loc", $mui_link;
		if($ret == 0){
			return 0;
		}
	} 
	return 1;
}

# main program

if($debuglevel!=0){
	open_log;
}
SWITCH:{
	(!defined($ARGV[0])) && do{
		print "Invalid usage of splxhttpd command.\n";
		print "Valid syntax is: '$0 {start|stop|restart|status}'.\n";
		print "Enter the correct splxhttpd command and try again.\n";
		exit 1;
				};
	($ARGV[0] eq "start") && do {
				#print "Starting Trend Micro NAS Security httpd: \n";
				#write_log "Starting Trend Micro NAS Security httpd:";

				if (!stat($splx_httpd_log_path)) {
					mkpath($splx_httpd_log_path, mode=>0777);
					($login,$pass,$uid,$gid) = getpwnam('tmhttpd');
					chown($uid, $gid, $var_path . '/httpd');
					chown($uid, $gid, $var_path . '/httpd/logs');
				}

				# create subsys directory
				if (! -d $subsys_path) {
					mkpath($subsys_path, mode=>0777);
				}

				# start splxhttpd
				print "Starting splxhttpd: ";
				install_logrotate(1);
				write_log "step: starting splxhttpd service.";
				%property=is_running "splxhttpd";
				if(!$property{"running"}==1)
				{
					my $loc = get_locale $locale_file;
					$ENV{'TM_LOCALE'} = $loc;
					if((make_mui_links $loc) == 0){
						echo_failure;
						write_log "error: failed to create links. locale=$loc";
						write_log "error: splxhttpd service can't be started";
						exit(1);
					}
					set_default_locale_env;
					if((system "$httpd_prog -f $httpd_conf_file > /dev/null 2>&1")!=0){
					#if((system "$httpd_prog -f $httpd_conf_file")!=0){
						echo_failure;
						info_failure("Some errors were found while starting splxhttpd. See the system log for error description. The default system log file path is /var/log/messages.\n");
						write_log "error: splxhttpd service can't be started.";
						exit(1);
					}	
				}
				echo_success;

				touch "/var/lock/subsys/splxhttpd";
				info_success( "Trend Micro NAS Security httpd started.\n");
				write_log "Trend Micro NAS Security httpd started.";

				last SWITCH;
				};
	($ARGV[0] eq "stop") && do {
				my $STOP_splxhttpd=0;
				my $no_splxhttpd=0;

        			#print "Shutting down Trend Micro NAS Security httpd:\n";
        			#write_log "Shutting down Trend Micro NAS Security httpd: ";

				# stop splxhttpd
				print "Shutting down splxhttpd: ";
				install_logrotate(0);
				  write_log "step: shutting down splxhttpd service.";
				  %property=is_running "splxhttpd";
				  if($property{"running"}==0){
				    unlink "/var/lock/subsys/splxhttpd";
				    write_log "info: splxhttpd service not running.";
				    echo_failure;
				    $STOP_splxhttpd=0;
				    $no_splxhttpd=1;
				  } else {
                                    $i=0;
				    while ($property{"running"} && $i<15){
				      if(! kill_proc "splxhttpd", "$httpd_pid_file"){
					write_log "Unable to stop splxhttpd. Trend Micro NAS Security will attempt to stop splxhttpd.";
				      }
				      sleep 1;
				      %property=is_running "splxhttpd";
                                      $i=$i+1;
				    }
                                    if($property{"running"}){
                                        write_log "Some errors were found while stopping splxhttpd.";
                                        echo_failure;
                                        $STOP_splxhttpd=1;
                                    }else{
				        unlink "/var/lock/subsys/splxhttpd";
                        unlink "$product_root/SPLX.tmp/cookies_tmp";
				        write_log "success: splxhttpd service stopped.";
				        echo_success;
				        $STOP_splxhttpd=0;
                                    }
				  }

				$STOP=$STOP_splxhttpd;
                                if($STOP==0){
                                				if($no_splxhttpd==1)
                                				{
                                        	info_failure( "Trend Micro NAS Security httpd not running.\n");
                                        	unlink "/var/lock/subsys/splxhttpd";
                                        	write_log "Trend Micro NAS Security httpd not running.";
                                        	exit 2;
                                				}
                                				
                                        info_success( "Trend Micro NAS Security httpd stopped normally.\n");
                                        unlink "/var/lock/subsys/splxhttpd";
                                        write_log "Trend Micro NAS Security httpd stopped.";
                                        exit 0;
                                }
                                else{
                                    info_failure("splxhttp stopped with errors. See the system log for error description. The default system log file path is /var/log/messages.\n");
                                         write_log "Trend Micro NAS Security httpd stopped with errors.";
                                        exit 1;
                                }
				last SWITCH;
				};
	($ARGV[0] eq "restart") && do {
				system "$0 stop";
				system "$0 start";
				last SWITCH;
				};
	($ARGV[0] eq "status") && do {
                                my $STATUS_splxhttpd=0;
				$STATUS_splxhttpd=show_status "splxhttpd";
				exit($STATUS_splxhttpd);
				last SWITCH;
				};
	print "Invalid usage of splxhttpd command.\n";
        print "Valid syntax is: '$0 {start|stop|restart|status}'.\n";
        print "Enter the correct splxhttpd command and try again.\n";
        
}

if($debuglevel!=0){
	close_log;
}

exit 0;
